/* t.c
 * A program to use module stats.o to measure timer stability.
 *
 * Author:	Eyal Lebedinsky (eyal@ise.canberra.edu.au)
 * Last update:	25 June 1995
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <ctype.h>
#include <sys/ioctl.h>

#include "stats.h"

static unsigned long	array[ST_ARRAY_SIZE];
static unsigned long	total[ST_ARRAY_SIZE];
static int		fd = -1;

void
die (char *str, int exitcode)
{
	fprintf (stderr, "\r%s failed: %s   \n", str, strerror (errno));
	if (fd >= 0) {
		close (fd);
		fd = -1;
	}
	exit (exitcode);
}

int
main (int argc, char *argv[])
{
	int		i, j;
	unsigned long	count = ST_DEF_COUNT;
	unsigned long	repeat = 1000;
	int		mode = ST_DEF_MODE;
	long		l;
	int		errs;
	int		verbose;

	errs = 0;
	verbose = 0;
	for (i = 1; i < argc; ++i) {
		if (!strcmp ("timer", argv[i]))
			mode = ST_MODE_TIMER;
		else if (!strcmp ("tod", argv[i]))
			mode = ST_MODE_TOD;
		else if ('c' == argv[i][0]) {
			if (1 != sscanf (argv[i]+1, "%lu", &l)
							|| l <= 0) {
				printf ("bad count '%s'\n", argv[i]);
				++errs;
			} else
				count = l;
		} else if (!strcmp ("v", argv[i]))
			++verbose;
		else if ('r' == argv[i][0]) {
			if (1 != sscanf (argv[i]+1, "%lu", &l)
							|| l <= 0) {
				printf ("bad repeat '%s'\n", argv[i]);
				++errs;
			} else
				repeat = l;
		} else {
			printf ("unknown argument '%s'\n", argv[i]);
			++errs;
		}
	}

	if (errs) {
		printf ("usage: %s [timer|tod] [rnnn] [cnnn] [v]\n", argv[0]);
		exit (1);
	}

	if ((fd = open ("/dev/test", O_RDONLY)) < 0)
		die ("open", -1);

	if (ioctl (fd, ST_SET_COUNT, &count) < 0)
		die ("ST_SET_COUNT", -1);
	printf ("count is %lu\n", count);
	printf ("repeat is %lu\n", repeat);

	if (ioctl (fd, ST_SET_MODE, &mode) < 0)
		die ("ST_SET_MODE", -1);
	printf ("mode is %s\n",
		ST_MODE_TIMER == mode ? "timer" : "tod");

	if (ioctl (fd, ST_CLEAR, NULL) < 0)
		die ("ST_CLEAR", -1);

	for (j = 0; j < ST_ARRAY_SIZE; ++j)
		total[j] = 0;

	for (i = 0; i < repeat;) {
		if (read (fd, array, sizeof (array)) != sizeof (array))
			die ("read", -1);
		else {
			for (j = 0; j < ST_ARRAY_SIZE; ++j)
				total[j] += array[j];
		}
		if (!(++i % 100) && verbose) {
			if (!(i % (100*50)))
				fprintf (stderr, "\n%u", i);
			fprintf (stderr, ".");
		}
	}
	if (verbose)
		fprintf (stderr, "\n");

	printf ("\n%5s %9s\n", "delay", "count");
	for (j = 0; j < ST_ARRAY_SIZE; ++j) {
		if (array[j])
			printf ("%5u %9lu\n", j, array[j]);
	}

	close (fd);
	fd = -1;

	exit (0);
	return (0);
}
